import { Injectable } from '@nestjs/common';
import { PermissionTypeRepository } from '../../modules/permission/repositories';
import { PermissionType } from '../../constants/permission-type.enum';

@Injectable()
export class PermissionTypeSeeder {
  constructor(
    private readonly permissionTypeRepository: PermissionTypeRepository,
  ) {}

  async addPermissionTypeDataIfNotFound(): Promise<void> {
    try {
      const existingPermissionTypes =
        await this.permissionTypeRepository.find();

      const permissionTypeNames = existingPermissionTypes.map((pt) => pt.type);

      const missingPermissionTypes = [];
      if (!permissionTypeNames.includes(PermissionType.CONTROLLABLE)) {
        missingPermissionTypes.push(PermissionType.CONTROLLABLE);
      }
      if (!permissionTypeNames.includes(PermissionType.READ)) {
        missingPermissionTypes.push(PermissionType.READ);
      }

      if (missingPermissionTypes.length > 0) {
        await this.addPermissionTypeData(missingPermissionTypes);
      }
    } catch (err) {
      console.error('Error while checking permission type data:', err);
      throw err;
    }
  }

  private async addPermissionTypeData(
    permissionTypes: string[],
  ): Promise<void> {
    try {
      const permissionTypeEntities = permissionTypes.map((type) => ({
        type,
      }));

      await this.permissionTypeRepository.save(permissionTypeEntities);
    } catch (err) {
      console.error('Error while adding permission type data:', err);
      throw err;
    }
  }
}
