import { Injectable } from '@nestjs/common';
import { RoleType } from '../../constants/role.type.enum';
import { RoleTypeRepository } from '../../modules/role-type/repositories';

@Injectable()
export class RoleTypeSeeder {
  constructor(private readonly roleTypeRepository: RoleTypeRepository) {}

  async addRoleTypeDataIfNotFound(): Promise<void> {
    try {
      const existingRoleTypes = await this.roleTypeRepository.find();

      const roleTypeNames = existingRoleTypes.map((pt) => pt.type);

      const missingRoleTypes = [];
      if (!roleTypeNames.includes(RoleType.SUPER_ADMIN)) {
        missingRoleTypes.push(RoleType.SUPER_ADMIN);
      }
      if (!roleTypeNames.includes(RoleType.ADMIN)) {
        missingRoleTypes.push(RoleType.ADMIN);
      }

      if (missingRoleTypes.length > 0) {
        await this.addRoleTypeData(missingRoleTypes);
      }
    } catch (err) {
      console.error('Error while checking role type data:', err);
      throw err;
    }
  }

  private async addRoleTypeData(roleTypes: string[]): Promise<void> {
    try {
      const roleTypeEntities = roleTypes.map((type) => ({
        type,
      }));

      await this.roleTypeRepository.save(roleTypeEntities);
    } catch (err) {
      console.error('Error while adding role type data:', err);
      throw err;
    }
  }
}
