import { SpacePermissionService } from '@app/common/helper/services/space.permission.service';
import {
  BadRequestException,
  CanActivate,
  ExecutionContext,
  Injectable,
} from '@nestjs/common';

@Injectable()
export class FloorPermissionGuard implements CanActivate {
  constructor(private readonly permissionService: SpacePermissionService) {}

  async canActivate(context: ExecutionContext): Promise<boolean> {
    const req = context.switchToHttp().getRequest();

    try {
      const { floorUuid } = req.params;
      const { user } = req;

      if (!floorUuid) {
        throw new BadRequestException('floorUuid is required');
      }

      await this.permissionService.checkUserPermission(
        floorUuid,
        user.uuid,
        'floor',
      );

      return true;
    } catch (error) {
      throw error;
    }
  }
}
