import { RoleTypeRepository } from './../../../libs/common/src/modules/role-type/repositories/role.type.repository';
import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { AddUserRoleDto } from '../dtos/role.add.dto';
import { UserRoleRepository } from '@app/common/modules/user-role/repositories';
import { QueryFailedError } from 'typeorm';

@Injectable()
export class RoleService {
  constructor(
    private readonly roleTypeRepository: RoleTypeRepository,
    private readonly userRoleRepository: UserRoleRepository,
  ) {}

  async addUserRoleType(addUserRoleDto: AddUserRoleDto) {
    try {
      const roleType = await this.fetchRoleByType(addUserRoleDto.roleType);

      if (roleType.uuid) {
        return await this.userRoleRepository.save({
          user: { uuid: addUserRoleDto.userUuid },
          roleType: { uuid: roleType.uuid },
        });
      }
    } catch (error) {
      if (
        error instanceof QueryFailedError &&
        error.driverError.code === '23505'
      ) {
        // Postgres unique constraint violation error code
        throw new HttpException(
          'This role already exists for this user',
          HttpStatus.CONFLICT,
        );
      }
      throw new HttpException(
        error.message || 'Internal Server Error',
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  async fetchRoleTypes() {
    const roleTypes = await this.roleTypeRepository.find();

    return roleTypes;
  }
  private async fetchRoleByType(roleType: string) {
    return await this.roleTypeRepository.findOne({
      where: {
        type: roleType,
      },
    });
  }
}
