import { ApiProperty } from '@nestjs/swagger';
import {
  IsNotEmpty,
  IsString,
  IsArray,
  ValidateNested,
  IsOptional,
  IsNumber,
} from 'class-validator';
import { Type } from 'class-transformer';

class ExecutorProperty {
  @ApiProperty({
    description: 'Function code (for device issue action)',
    required: false,
  })
  @IsString()
  @IsOptional()
  public functionCode?: string;

  @ApiProperty({
    description: 'Function value (for device issue action)',
    required: false,
  })
  @IsOptional()
  public functionValue?: any;

  @ApiProperty({
    description: 'Delay in seconds (for delay action)',
    required: false,
  })
  @IsNumber()
  @IsOptional()
  public delaySeconds?: number;
}

class Action {
  @ApiProperty({
    description: 'Entity ID',
    required: true,
  })
  @IsString()
  @IsNotEmpty()
  public entityId: string;

  @ApiProperty({
    description: 'Action executor',
    required: true,
  })
  @IsString()
  @IsNotEmpty()
  public actionExecutor: string;

  @ApiProperty({
    description: 'Executor property',
    required: false, // Set required to false
    type: ExecutorProperty,
  })
  @ValidateNested()
  @Type(() => ExecutorProperty)
  @IsOptional() // Make executorProperty optional
  public executorProperty?: ExecutorProperty;
}

export class AddSceneTapToRunDto {
  @ApiProperty({
    description: 'Unit UUID',
    required: true,
  })
  @IsString()
  @IsNotEmpty()
  public unitUuid: string;

  @ApiProperty({
    description: 'Scene name',
    required: true,
  })
  @IsString()
  @IsNotEmpty()
  public sceneName: string;

  @ApiProperty({
    description: 'Decision expression',
    required: true,
  })
  @IsString()
  @IsNotEmpty()
  public decisionExpr: string;

  @ApiProperty({
    description: 'Actions',
    required: true,
    type: [Action],
  })
  @IsArray()
  @ValidateNested({ each: true })
  @Type(() => Action)
  public actions: Action[];

  constructor(dto: Partial<AddSceneTapToRunDto>) {
    Object.assign(this, dto);
  }
}
export class UpdateSceneTapToRunDto {
  @ApiProperty({
    description: 'Scene name',
    required: true,
  })
  @IsString()
  @IsNotEmpty()
  public sceneName: string;

  @ApiProperty({
    description: 'Decision expression',
    required: true,
  })
  @IsString()
  @IsNotEmpty()
  public decisionExpr: string;

  @ApiProperty({
    description: 'Actions',
    required: true,
    type: [Action],
  })
  @IsArray()
  @ValidateNested({ each: true })
  @Type(() => Action)
  public actions: Action[];

  constructor(dto: Partial<AddSceneTapToRunDto>) {
    Object.assign(this, dto);
  }
}
